@@ -7,12 +7,12 @@ from pywayland.server import Display
 from wlroots.allocator import Allocator
 from wlroots.backend import Backend, BackendType
 from wlroots.renderer import Renderer
-from wlroots.wlr_types import Compositor
+from wlroots.wlr_types import Compositor, SubCompositor
 
 
 def build_compositor(
     display: Display, *, backend_type=BackendType.AUTO
-) -> tuple[Compositor, Allocator, Renderer, Backend]:
+) -> tuple[Compositor, Allocator, Renderer, Backend, SubCompositor]:
     """Build and run a compositor
 
     :param display:
@@ -29,5 +29,6 @@ def build_compositor(
     renderer.init_display(display)
     allocator = Allocator.autocreate(backend, renderer)
     compositor = Compositor(display, renderer)
+    subcompositor = SubCompositor(display)
 
-    return compositor, allocator, renderer, backend
+    return compositor, allocator, renderer, backend, subcompositor
