@@ -1,31 +1,35 @@
 # Copyright (c) Sean Vig 2019
+from __future__ import annotations
 
 import enum
-from typing import Optional, Tuple
 
 from pywayland.server import Signal
 
-from wlroots import ffi, PtrHasData, lib
+from wlroots import PtrHasData, ffi, lib
+
 from .input_device import InputDevice, InputDeviceType
+from .output import Output
 from .output_layout import OutputLayout
 from .pointer import (
-    PointerEventAxis,
-    PointerEventButton,
-    PointerEventMotion,
-    PointerEventMotionAbsolute,
-    PointerEventPinchBegin,
-    PointerEventPinchEnd,
-    PointerEventPinchUpdate,
-    PointerEventSwipeBegin,
-    PointerEventSwipeEnd,
-    PointerEventSwipeUpdate,
+    PointerAxisEvent,
+    PointerButtonEvent,
+    PointerHoldBeginEvent,
+    PointerHoldEndEvent,
+    PointerMotionAbsoluteEvent,
+    PointerMotionEvent,
+    PointerPinchBeginEvent,
+    PointerPinchEndEvent,
+    PointerPinchUpdateEvent,
+    PointerSwipeBeginEvent,
+    PointerSwipeEndEvent,
+    PointerSwipeUpdateEvent,
 )
 from .surface import Surface
 from .touch import (
-    TouchEventUp,
+    TouchEventCancel,
     TouchEventDown,
     TouchEventMotion,
-    TouchEventCancel,
+    TouchEventUp,
 )
 
 
@@ -52,44 +56,51 @@ class Cursor(PtrHasData):
         lib.wlr_cursor_attach_output_layout(self._ptr, output_layout._ptr)
 
         self.motion_event = Signal(
-            ptr=ffi.addressof(self._ptr.events.motion), data_wrapper=PointerEventMotion
+            ptr=ffi.addressof(self._ptr.events.motion), data_wrapper=PointerMotionEvent
         )
         self.motion_absolute_event = Signal(
             ptr=ffi.addressof(self._ptr.events.motion_absolute),
-            data_wrapper=PointerEventMotionAbsolute,
+            data_wrapper=PointerMotionAbsoluteEvent,
         )
         self.button_event = Signal(
-            ptr=ffi.addressof(self._ptr.events.button), data_wrapper=PointerEventButton
+            ptr=ffi.addressof(self._ptr.events.button), data_wrapper=PointerButtonEvent
         )
         self.axis_event = Signal(
-            ptr=ffi.addressof(self._ptr.events.axis), data_wrapper=PointerEventAxis
+            ptr=ffi.addressof(self._ptr.events.axis), data_wrapper=PointerAxisEvent
         )
         self.frame_event = Signal(ptr=ffi.addressof(self._ptr.events.frame))
         self.swipe_begin = Signal(
             ptr=ffi.addressof(self._ptr.events.swipe_begin),
-            data_wrapper=PointerEventSwipeBegin,
+            data_wrapper=PointerSwipeBeginEvent,
         )
         self.swipe_update = Signal(
             ptr=ffi.addressof(self._ptr.events.swipe_update),
-            data_wrapper=PointerEventSwipeUpdate,
+            data_wrapper=PointerSwipeUpdateEvent,
         )
         self.swipe_end = Signal(
             ptr=ffi.addressof(self._ptr.events.swipe_end),
-            data_wrapper=PointerEventSwipeEnd,
+            data_wrapper=PointerSwipeEndEvent,
         )
         self.pinch_begin = Signal(
             ptr=ffi.addressof(self._ptr.events.pinch_begin),
-            data_wrapper=PointerEventPinchBegin,
+            data_wrapper=PointerPinchBeginEvent,
         )
         self.pinch_update = Signal(
             ptr=ffi.addressof(self._ptr.events.pinch_update),
-            data_wrapper=PointerEventPinchUpdate,
+            data_wrapper=PointerPinchUpdateEvent,
         )
         self.pinch_end = Signal(
             ptr=ffi.addressof(self._ptr.events.pinch_end),
-            data_wrapper=PointerEventPinchEnd,
+            data_wrapper=PointerPinchEndEvent,
+        )
+        self.hold_begin = Signal(
+            ptr=ffi.addressof(self._ptr.events.hold_begin),
+            data_wrapper=PointerHoldBeginEvent,
+        )
+        self.hold_end = Signal(
+            ptr=ffi.addressof(self._ptr.events.hold_end),
+            data_wrapper=PointerHoldEndEvent,
         )
-
         self.touch_up_event = Signal(
             ptr=ffi.addressof(self._ptr.events.touch_up),
             data_wrapper=TouchEventUp,
@@ -139,9 +150,9 @@ class Cursor(PtrHasData):
             InputDeviceType.TOUCH,
             InputDeviceType.TABLET_TOOL,
         )
-        if input_device.device_type not in allowed_device_types:
+        if input_device.type not in allowed_device_types:
             raise ValueError(
-                f"Input device must be one of pointer, touch, or tablet tool, got: {input_device.device_type}"
+                f"Input device must be one of pointer, touch, or tablet tool, got: {input_device.type}"
             )
 
         lib.wlr_cursor_attach_input_device(self._ptr, input_device._ptr)
@@ -151,7 +162,7 @@ class Cursor(PtrHasData):
         delta_x: float,
         delta_y: float,
         *,
-        input_device: Optional[InputDevice] = None,
+        input_device: InputDevice | None = None,
     ) -> None:
         """Move the cursor in the direction of the given x and y layout coordinates
 
@@ -170,10 +181,10 @@ class Cursor(PtrHasData):
     def warp(
         self,
         warp_mode: WarpMode,
-        x: Optional[float],
-        y: Optional[float],
+        x: float | None,
+        y: float | None,
         *,
-        input_device: Optional[InputDevice] = None,
+        input_device: InputDevice | None = None,
     ) -> bool:
         """Warp the cursor to the given x and y in location
 
@@ -213,8 +224,8 @@ class Cursor(PtrHasData):
             raise ValueError("Invalid warp mode")
 
     def absolute_to_layout_coords(
-        self, input_device: Optional[InputDevice], x: float, y: float
-    ) -> Tuple[float, float]:
+        self, input_device: InputDevice | None, x: float, y: float
+    ) -> tuple[float, float]:
         """Convert absolute 0..1 coordinates to layout coordinates
 
         The `input_device` may be passed to respect device mapping constraints.
@@ -233,7 +244,7 @@ class Cursor(PtrHasData):
 
         return xy_ptr[0], xy_ptr[1]
 
-    def set_surface(self, surface: Optional[Surface], hotspot: Tuple[int, int]) -> None:
+    def set_surface(self, surface: Surface | None, hotspot: tuple[int, int]) -> None:
         """Set the cursor surface
 
         The surface can be committed to update the cursor image. The surface
@@ -247,10 +258,38 @@ class Cursor(PtrHasData):
 
         lib.wlr_cursor_set_surface(self._ptr, surface_ptr, hotspot[0], hotspot[1])
 
-    def __enter__(self) -> "Cursor":
+    def __enter__(self) -> Cursor:
         """Context manager to clean up the cursor"""
         return self
 
     def __exit__(self, exc_type, exc_value, exc_tb) -> None:
         """Clean up the cursor when exiting the context"""
         self.destroy()
+
+    def map_to_output(self, output: Output | None) -> None:
+        """
+        Attaches this cursor to the given output, which must be among the outputs in the
+        current output_layout for this cursor. This call is invalid for a cursor without
+        an associated output layout.
+        """
+        if output is None:
+            output_ptr = ffi.NULL
+        else:
+            output_ptr = output._ptr
+
+        lib.wlr_cursor_map_to_output(self._ptr, output_ptr)
+
+    def map_input_to_output(
+        self, input_device: InputDevice, output: Output | None
+    ) -> None:
+        """
+        Maps all input from a specific input device to a given output. The input device
+        must be attached to this cursor and the output must be among the outputs in the
+        attached output layout.
+        """
+        if output is None:
+            output_ptr = ffi.NULL
+        else:
+            output_ptr = output._ptr
+
+        lib.wlr_cursor_map_input_to_output(self._ptr, input_device._ptr, output_ptr)
